// AddTest.cpp : Defines the entry point for the console application.
//


#include "stdafx.h"

#include "..\Adder.h"

#include <comdef.h>

#include "..\StrUtils.h"
#include "TranString.h"

#define USEPTR(X) _COM_SMARTPTR_TYPEDEF(X,__uuidof(X))

USEPTR(IOpenTool);
USEPTR(IOTDescribe);
USEPTR(IOTChain);
USEPTR(IOTDataVector);
USEPTR(IEgOTDataOptReal);

#include <iostream>

using namespace std;

#define TEST(X) do { if (!bool(X)) { cout<<"FAIL "<<##X<<endl; break; } } while (0)
#define COMTEST(X) do { HRESULT h=(X); if (FAILED(h)) { cout<<"FAIL "<<##X<<":"<<hex<<(h)<<endl; break; } } while (0)

int main(int p_nArgs,char *p_asArgs[])
{
	HRESULT t_hInit=CoInitialize(0);
	if (FAILED(t_hInit))
		return t_hInit;

	do
	{

		// Get a pointer to the main tool
		IOpenToolPtr t_spIOTTrivialAdder(__uuidof(TrivialAdder));		TEST(t_spIOTTrivialAdder);

		// Get a pointer to one real number outputting tool
 		IOpenToolPtr t_spIOTStubReal1(__uuidof(StubReal));				TEST(t_spIOTStubReal1);

		// Get a pointer to anoter real number outputting tool
		IOpenToolPtr t_spIOTStubReal2(__uuidof(StubReal));				TEST(t_spIOTStubReal2);

		// Get some description info from the main tool
		IOTDescribePtr t_spIOTDescribe(t_spIOTTrivialAdder);			TEST(t_spIOTDescribe);

		// Get its name
		wchar_t* t_pwsX;
		wchar_t* t_pwsY;
		COMTEST(t_spIOTDescribe->Name(&t_pwsX,&t_pwsY));
		cout<< (const char*)TranString(t_pwsX) <<" [Aliases: "<<(const char*)TranString(t_pwsY)<<"]" <<endl;
		StrFree(&t_pwsX);
		StrFree(&t_pwsY);

		// Get the tool's chaining interface
		IOTChainPtr t_spIOTChain(t_spIOTTrivialAdder);					TEST(t_spIOTChain);

		// Chain in one real
		COMTEST(t_spIOTChain->ChainIn(t_spIOTStubReal1));

		// Chain in another
		COMTEST(t_spIOTChain->ChainIn(t_spIOTStubReal2));

		// Get the output interface
		IEgOTDataOptRealPtr t_spIEgOTDataOptReal(t_spIOTTrivialAdder);	TEST(t_spIEgOTDataOptReal);

		// Ask it for the sum
		real t_rSum=0.;
		COMTEST(t_spIEgOTDataOptReal->EgOptReal(&t_rSum));
		cout<<"Sum is: "<<t_rSum<<endl;

		IOTDataVectorPtr t_spIOTDataVector(t_spIOTTrivialAdder);	TEST(t_spIOTDataVector);

		t_rSum=0.;
		real t_rRead=0;
		index t_iStart=0;
		unsigned t_uReturned=0;
		index t_iRemain=0;
		COMTEST(t_spIOTDataVector->Vector(&t_rRead,1U,&t_iStart,&t_uReturned,&t_iRemain));	// Get the summand (ignore more than 1)
		if (t_uReturned)
			t_rSum+=t_rRead;	// Add it up

		char t_acBuffer[10];

		cout
		<<	"t_rRead="<<t_rRead
		<<	", t_iStart="<<_ui64toa(t_iStart,t_acBuffer,10)
		<<	", t_uReturned="<<t_uReturned
		<<	", t_iRemain="<<_ui64toa(t_iRemain,t_acBuffer,10)
		<<	", t_rSum="<<t_rSum
		<<endl;


// END OF TEST
	
		cout<<"OK"<<endl;
	}
	while (0);

	CoUninitialize();
	cout<<"\nEND."<<endl;
	return 0;
}
