  //      TECHNICAL NOTES & DESCRIPTION:  
    //      
    //      A macro to enable reference count debugging in your ATL coclass.
    //      
    //      ATL already has a refcount debugging macro: _ATL_DEBUG_REFCOUNT.  
    //      Contrary to what the docs imply, this macro only enables refcount 
    //      debugging for standard ATL classes.  NOT for your coclasses.  
    //      _DEBUG_REFCOUNT_IMPL does enable refcount debugging for your 
    //      coclasses, but you must use the macro once for each coclass you 
    //      want to debug.
    //      
    //      Use:
    //      1) #define _ATL_DEBUG_REFCOUNT in the project settings.
    //      2) use the _DEBUG_REFCOUNT_IMPL macro at the end of the COM map for
    //         your coclass:
    //      
    //          DECLARE_REGISTRY_RESOURCEID(IDR_FOO)
    //          
    //          BEGIN_COM_MAP(CFoo)
    //              COM_INTERFACE_ENTRY(IFoo)
    //              COM_INTERFACE_ENTRY(IDispatch)
    //              COM_INTERFACE_ENTRY(ISupportErrorInfo)
    //          END_COM_MAP()
    //          
    //          _DEBUG_REFCOUNT_IMPL(CFoo,CComSingleThreadModel)
    //      
    //      3) Watch the messages in the debug output window.
    //      
    //      This macro was adapted from work by Stephen W Gissendaner 
    //       that was posted to the ATL 
    //      maillist, 8 Jul 1997.  Ben Hickman  added the 
    //      threadingmodel parameter and massaged the code into building.
    //	
    //*****************************************************************************

    #ifdef _ATL_DEBUG_REFCOUNT
        #define _DEBUG_REFCOUNT_IMPL(classname,threadingmodel)\
            public:\
            ULONG InternalAddRef(void)\
            {\
                ULONG ulReturn;\
                ulReturn = CComObjectRootEx<threadingmodel>::InternalAddRef();\
                ATLTRACE(_T("+++ AddRef()     %s::%p  refcount = %d +++\n"), _T(#classname), this, m_dwRef);\
                return ulReturn;\
            }\
            ULONG InternalRelease(void)\
            {\
                ULONG ulReturn;\
                ulReturn = CComObjectRootEx<threadingmodel>::InternalRelease();\
                ATLTRACE(_T("--- Release()    %s::%p  refcount = %d ---\n"), _T(#classname), this, m_dwRef);\
                return ulReturn;\
            }
    #else
    #define _DEBUG_REFCOUNT_IMPL(classname,threadingmodel)
    #endif
	
