// OutputSum.cpp : Implementation of CAdderApp and DLL registration.

#include "stdafx.h"
#include "Adder.h"
#include "OutputSum.h"

#include "StrUtils.h"
#include "MacroUtils.h"

/////////////////////////////////////////////////////////////////////////////
//

#define ONFAIL(X,E) if (FAILED(X)) return (E)
#define ONFAILR(X) do { HRESULT t_h=(X); if (FAILED(t_h)) return (t_h); } while (0)
#define EXPECT(X) ONFAIL((X),E_UNEXPECTED)

// IOTConnector
STDMETHODIMP OutputSum::OpenTool(IOpenTool **a_ppIOpenTool)
{	if (!a_ppIOpenTool)
		return E_POINTER;

	*a_ppIOpenTool=0;

	return GetControllingUnknown()->QueryInterface(IOUT(a_ppIOpenTool));
}

STDMETHODIMP OutputSum::ConnectorType()
{	return e_ConnectorPrimary;
}


// IOTConnectorDescribe
STDMETHODIMP OutputSum::ConnectorName(wchar_t **a_pwsName, wchar_t **a_pwsAliases)
{	if (!a_pwsName || !a_pwsAliases)
		return E_POINTER;

	*a_pwsAliases=0;

	return StrDup(a_pwsName,L"Sum");
}

STDMETHODIMP OutputSum::ConnectorDescription(wchar_t **a_pwsDescription)
{	return StrDup(a_pwsDescription,L"The total of the two inputs");
}

// IOTOutput

STDMETHODIMP OutputSum::Status()
{	
	CComPtr<IOpenTool> t_spIOpenTool;

	EXPECT(OpenTool(&t_spIOpenTool));	// Our parent // We should be part of an open tool

	CComPtr<IEnumUnknown> t_spIEnumUnknown;
	EXPECT(t_spIOpenTool->QueryInterface(&t_spIEnumUnknown));	// We want to find our inputs

	EXPECT(t_spIEnumUnknown->Reset());	// Start looking at each potential connector

	unsigned t_uInputs=0;
	unsigned t_uAvailable=0;

	do
	{	CComPtr<IUnknown> t_spIUnknown;
		ULONG t_ulFetched=0; 
		EXPECT(t_spIEnumUnknown->Next(1UL,&t_spIUnknown,&t_ulFetched));	// Fetch potential connector
		if (!t_ulFetched)
			break;


		CComQIPtr<IOTInput> t_spIOTInput(t_spIUnknown);
		if (t_spIOTInput)	// If it is an input
		{	CComPtr<IOTConnector> t_spIOTConnector;
			EXPECT(t_spIOTInput->Input(&t_spIOTConnector));	// Ask for the connected connector

			if (t_spIOTConnector)	// Is there an attached connector?
			{	CComQIPtr<IOTOutput> t_spIOTOutput(t_spIOTConnector);
				if (t_spIOTOutput)	// Hopefully it's an output
					if (CComQIPtr<IEgOTDataOptReal>(t_spIOTOutput) || CComQIPtr<IOTDataVector>(t_spIOTOutput))	// We're only interested in things we can add up
					{	++t_uInputs;

						HRESULT t_hStatus=t_spIOTOutput->Status();
						if (FAILED(t_hStatus))
							return E_UNEXPECTED;

						t_uAvailable+=!t_hStatus;	// Count complete inputs
					}
			}
			
		}


	}
	while (true);

	bool t_bAvailable=true;											// Sum may be accessed immediately
	bool t_bEmpty=!t_uAvailable;									// Empty if no inputs
	bool t_bPartial=t_uAvailable==1u;								// Partial if one input
	bool t_bFull=t_uAvailable>=2u;									// Full if at least 2 available inputs
	bool t_bComplete=t_uInputs>=2u && t_uInputs==t_uAvailable;		// Complete if at least 2 inputs and all available

	return
		(t_bComplete?e_StatusComplete:e_StatusIncomplete)
		|	(t_bFull?e_StatusFull:(t_bPartial?e_StatusPartial:e_StatusEmpty))
		|	(t_bAvailable?e_StatusAvailable:e_StatusUnavailable)
	;
}

STDMETHODIMP OutputSum::Changes(index* a_piState)
{
	if (a_piState && *a_piState)
		return E_INVALIDARG;	// We only return 0

	return S_OK;	// non-zero if has changed
}

// IOTDataVector

STDMETHODIMP OutputSum::Nature()
{	return S_OK;
}

STDMETHODIMP OutputSum::Vector(real *a_prBuffer, unsigned a_uBufferLength, index* a_piStart, unsigned* a_puReturned,index* a_piRemain)
{
	if (!a_piStart || !a_puReturned || !a_piRemain)
		return E_POINTER;

	if (a_prBuffer && a_uBufferLength && !*a_piStart)	// Were we given a buffer and asked for the zeroth real?
	{	EXPECT(EgOptReal(a_prBuffer));	// Get the sum

		*a_puReturned=1U;	// 1 off
		*a_piRemain=0;		// None left
	}
	else
	{	*a_puReturned=0;	// Couldn't return anything
		*a_piRemain=1UL;	// 1 real left
	}

	*a_piStart=0;			// 0 is always position of next real (non-zero or otherwise)

	return S_OK;
}


// IEgOTDataOptReal

STDMETHODIMP OutputSum::EgOptNature()
{	return S_OK;
}

STDMETHODIMP OutputSum::EgOptReal(real *a_prValue)
{
	CComPtr<IOpenTool> t_spIOpenTool;

	EXPECT(OpenTool(&t_spIOpenTool));	// Our parent // We should be part of an open tool

	CComPtr<IEnumUnknown> t_spIEnumUnknown;
	EXPECT(t_spIOpenTool->QueryInterface(&t_spIEnumUnknown));	// We want to find our inputs

	EXPECT(t_spIEnumUnknown->Reset());	// Start looking at each potential connector

	real t_rSum=0.;

	do
	{	CComPtr<IUnknown> t_spIUnknown;
		ULONG t_ulFetched=0; 
		EXPECT(t_spIEnumUnknown->Next(1UL,&t_spIUnknown,&t_ulFetched));	// Fetch potential connector
		if (!t_ulFetched)
			break;


		CComQIPtr<IOTInput> t_spIOTInput(t_spIUnknown);
		if (t_spIOTInput)	// If it is an input
		{	CComPtr<IOTConnector> t_spIOTConnector;
			EXPECT(t_spIOTInput->Input(&t_spIOTConnector));	// Ask for the connected connector

			if (t_spIOTConnector)	// Is there an attached connector?
			{	CComQIPtr<IOTOutput> t_spIOTOutput(t_spIOTConnector);
				if (t_spIOTOutput)	// Hopefully it's an output
				{	CComQIPtr<IEgOTDataOptReal> t_spIEgOTDataOptReal(t_spIOTOutput);
					if (t_spIEgOTDataOptReal)	// We're only interested in things we can add up
					{	HRESULT t_hStatus=t_spIOTOutput->Status();	// Check its status
						if (FAILED(t_hStatus))
							return E_UNEXPECTED;

						if (!t_hStatus)	// If complete
						{	real t_rRead=0;
							EXPECT(t_spIEgOTDataOptReal->EgOptReal(&t_rRead));	// Get the summand
							t_rSum+=t_rRead;	// Add it up
						}
					}
					else
					{	CComQIPtr<IOTDataVector> t_spIOTDataVector(t_spIOTOutput);
						if (t_spIOTDataVector)	// We're only interested in things we can add up
						{	HRESULT t_hStatus=t_spIOTOutput->Status();	// Check its status
							if (FAILED(t_hStatus))
								return E_UNEXPECTED;

							if (!t_hStatus)	// If complete
							{	real t_rRead=0;
								index t_iStart=0;
								unsigned t_uReturned=0;
								index t_iRemain=0;
								EXPECT(t_spIOTDataVector->Vector(&t_rRead,1U,&t_iStart,&t_uReturned,&t_iRemain));	// Get the summand (ignore more than 1)
								if (t_uReturned)
									t_rSum+=t_rRead;	// Add it up
							}
						}
					}
				}
			}
			
		}


	}
	while (true);

	*a_prValue=t_rSum;	// Return the sum

	return S_OK;
}

