// TrivialAdder.cpp : Implementation of CAdderApp and DLL registration.

#include "stdafx.h"
#include "Adder.h"
#include "TrivialAdder.h"

#include "StrUtils.h"
#include "MacroUtils.h"

#include <set>

/////////////////////////////////////////////////////////////////////////////
//

HRESULT TrivialAdder::FinalConstruct()
{
	m_InputSummand1.SetOuterUnknown(GetControllingUnknown());
	m_InputSummand2.SetOuterUnknown(GetControllingUnknown());

	HRESULT t_h=m_spIUnknownOutputSum.CoCreateInstance(__uuidof(OutputSum),GetControllingUnknown(),CLSCTX_INPROC_SERVER);
	if (FAILED(t_h))
		return t_h;

	m_apIUnknown[0]=m_InputSummand1.GetUnknown();
	m_apIUnknown[1]=m_InputSummand2.GetUnknown();
	m_apIUnknown[2]=GetControllingUnknown();	// Our output is also exposed by this class

	CComObject<tEnum>::CreateInstance(&m_pEnumConnectors);	// Create an enumerator

	if (!m_pEnumConnectors)
		return E_OUTOFMEMORY;

	t_h=m_pEnumConnectors->Init(m_apIUnknown,m_apIUnknown+3,GetControllingUnknown(),AtlFlagNoCopy);	// Initialise it
	if (FAILED(t_h))
	{	delete m_pEnumConnectors;
		m_pEnumConnectors=0;
		return t_h;
	}
	GetControllingUnknown()->Release();	// Undo the AddRef by the Enumerator

	return S_OK;
}

void TrivialAdder::FinalRelease()
{	if (m_pEnumConnectors)					// Dump our EnumUnknown implementation
	{	GetControllingUnknown()->AddRef();	// Pre-increment, because the Enumerator will release us
		delete m_pEnumConnectors;
		m_pEnumConnectors=0;
	}

	m_spIUnknownOutputSum.Release();		// Dump our aggregated output
}


// IOpenTool

// IOTDescribe
STDMETHODIMP TrivialAdder::Name(wchar_t **a_pwsName, wchar_t **a_pwsAliases)
{	if (!a_pwsName || !a_pwsAliases)
		return E_POINTER;

	*a_pwsAliases=0;

	return StrDup(a_pwsName,L"TrivialAdder");
}


STDMETHODIMP TrivialAdder::Class(wchar_t **a_pwsClass, wchar_t **a_pwsAliases)
{	if (!a_pwsClass || !a_pwsAliases)
		return E_POINTER;

	*a_pwsAliases=0;

	return StrDup(a_pwsClass,L"Arithmetic");
}

STDMETHODIMP TrivialAdder::Description(wchar_t **a_pwsDescription)
{	return StrDup(a_pwsDescription,L"Adds two numbers");
}


// IOTChain

HRESULT TrivialAdder::ChainInOutput(IUnknown *a_pIUnknown)
{	if (!a_pIUnknown)
		return E_INVALIDARG;

	CComPtr<IOTConnector> t_spIOTConnectorOutput;
	if (FAILED(a_pIUnknown->QueryInterface(IOUT(&t_spIOTConnectorOutput))))	// Ask for the connected connector
		return S_OK;	// Ignore non-connectors
	
	CComPtr<IOTOutput> t_spIOTOutput;
	if (FAILED(t_spIOTConnectorOutput->QueryInterface(&t_spIOTOutput)))	// Ask for its output
		return S_OK;	// Ignore connectors without output

	CComPtr<IEnumUnknown> t_spIEnumUnknown;	// Pointer for our list of connectors (We don't access our native interface directly, just in case we've been aggregated)
	ONFAILR(GetControllingUnknown()->QueryInterface(IOUT(&t_spIEnumUnknown)));	// We know a priori we implement this
	EXPECT(t_spIEnumUnknown->Reset());	// Start looking at each potential output	

	long t_lConnected=0;

	do
	{	CComPtr<IUnknown> t_spIUnknown;
		ULONG t_ulFetched=0; 
		EXPECT(t_spIEnumUnknown->Next(1UL,&t_spIUnknown,&t_ulFetched));	// Fetch potential input
		if (!t_ulFetched)
			break;

		CComPtr<IOTInput> t_spIOTInput;
		if (SUCCEEDED(t_spIUnknown->QueryInterface(&t_spIOTInput)))	// If it is an input
		{	CComPtr<IOTConnector> t_spIOTConnectorInput;
			EXPECT(t_spIOTInput->Input(&t_spIOTConnectorInput));	// Ask for the connected connector

			if (!t_spIOTConnectorInput)	// Is it vacant?
			{
				HRESULT t_h=t_spIOTInput->Connect(t_spIOTConnectorOutput);	// Attempt to connect the output to this input of ours
				if (FAILED(t_h))
					return t_h;

				if (t_h&e_CompatibilityKnown && t_h&e_CompatibilityFailure)	// Is it incompatible?
					ONFAILR(t_spIOTInput->Connect(0));	// Disconnect
				else	
					return 1L;	// Successful connection
			}
		}
	}
	while (true);
	
	return 0;	// No inputs could be found that would accept the supplied output connector
}

STDMETHODIMP TrivialAdder::ChainIn(IOpenTool *a_pIOpenTool)
{	if (!a_pIOpenTool)
		return E_INVALIDARG;

// For each output on the supplied tool, see if we have a compatible vacant input

// If IEnumUnknown is supported on the tool, try those for IOTOutput, otherwise try QI

	long t_lCount=0;

	CComPtr<IEnumUnknown> t_spIEnumUnknown;
	if (SUCCEEDED(a_pIOpenTool->QueryInterface(IOUT(&t_spIEnumUnknown))))	// We want to find outputs
	{	EXPECT(t_spIEnumUnknown->Reset());	// Start looking at each potential output

		do
		{	CComPtr<IUnknown> t_spIUnknown;
			ULONG t_ulFetched=0; 
			EXPECT(t_spIEnumUnknown->Next(1UL,&t_spIUnknown,&t_ulFetched));	// Fetch potential output
			if (!t_ulFetched)
				break;

			HRESULT t_h=ChainInOutput(t_spIUnknown);
			if (FAILED(t_h))
				return t_h;
			
			t_lCount+=t_h;
		}
		while (true);
	}
	else
	{	HRESULT t_h=ChainInOutput(a_pIOpenTool);
		if (FAILED(t_h))
			return t_h;
		
		t_lCount+=t_h;
	}

	return t_lCount;
}

STDMETHODIMP TrivialAdder::Remove(IOpenTool *a_pIOpenTool)
{	if (!a_pIOpenTool)
		return E_INVALIDARG;

	// For all connected inputs, if its parent tool matches that supplied, disconnect it

	CComPtr<IUnknown> t_spIUnknownOpenToolTest;	// The interface we'll be comparing against
	EXPECT(a_pIOpenTool->QueryInterface(IOUT(&t_spIUnknownOpenToolTest)));	// Get it
		
	CComPtr<IEnumUnknown> t_spIEnumUnknown;	// Pointer for our list of connectors (We don't access our native interface directly, just in case we've been aggregated)
	ONFAILR(QueryInterface(IOUT(&t_spIEnumUnknown)));	// We know a priori we implement this
	EXPECT(t_spIEnumUnknown->Reset());	// Start looking at each potential output	

	long t_lCount=0;

	do
	{	CComPtr<IUnknown> t_spIUnknown;
		ULONG t_ulFetched=0; 
		EXPECT(t_spIEnumUnknown->Next(1UL,&t_spIUnknown,&t_ulFetched));	// Fetch potential input
		if (!t_ulFetched)
			break;

		CComPtr<IOTInput> t_spIOTInput;
		if (SUCCEEDED(t_spIUnknown->QueryInterface(&t_spIOTInput)))	// If it is an input
		{	CComPtr<IOTConnector> t_spIOTConnectorInput;
			EXPECT(t_spIOTInput->Input(&t_spIOTConnectorInput));	// Ask for the connected connector

			if (t_spIOTConnectorInput)	// Is it connected?
			{	
				CComPtr<IOpenTool> t_spIOpenTool;
				EXPECT(t_spIOTConnectorInput->OpenTool(&t_spIOpenTool));	// Ask for the input's OpenTool

				CComPtr<IUnknown> t_spIUnknownOpenToolInput;	// Get it's unknown
				EXPECT(t_spIOpenTool->QueryInterface(&t_spIUnknownOpenToolInput));	// Get it

				if (t_spIUnknownOpenToolInput==t_spIUnknownOpenToolTest)	// Does this input belong to the specified Open Tool?
				{	EXPECT(t_spIOTInput->Connect(0));	// Remove the connection
				
					++t_lCount;
				}
			}
		}
	}
	while (true);


	return t_lCount;
}

STDMETHODIMP TrivialAdder::RemoveAll()
{
	// Break all input connections

	// For all connected inputs, if its parent tool matches that supplied, disconnect it

	CComPtr<IEnumUnknown> t_spIEnumUnknown;	// Pointer for our list of connectors (We don't access our native interface directly, just in case we've been aggregated)
	ONFAILR(QueryInterface(IOUT(&t_spIEnumUnknown)));	// We know a priori we implement this
	EXPECT(t_spIEnumUnknown->Reset());	// Start looking at each potential output	

	do
	{	CComPtr<IUnknown> t_spIUnknown;
		ULONG t_ulFetched=0; 
		EXPECT(t_spIEnumUnknown->Next(1UL,&t_spIUnknown,&t_ulFetched));	// Fetch potential input
		if (!t_ulFetched)
			break;

		CComPtr<IOTInput> t_spIOTInput;
		if (SUCCEEDED(t_spIUnknown->QueryInterface(&t_spIOTInput)))	// If it is an input
			EXPECT(t_spIOTInput->Connect(0));	// Ensure the connection is broken
	}
	while (true);

	return S_OK;
}

STDMETHODIMP TrivialAdder::Enum(IEnumUnknown **a_ppIEnumUnknown)
{	if (!a_ppIEnumUnknown)
		return E_INVALIDARG;

	*a_ppIEnumUnknown=0;

	// List all open tools connected to our inputs

	std::set<IUnknown*> t_setIUnknown;

	CComPtr<IEnumUnknown> t_spIEnumUnknown;	// Pointer for our list of connectors (We don't access our native interface directly, just in case we've been aggregated)
	ONFAILR(QueryInterface(IOUT(&t_spIEnumUnknown)));	// We know a priori we implement this

	HRESULT t_hReturn=S_OK;
	IUnknown** t_ppIUnknown=0;	// Pointer to interim copy of list of Open Tools
	unsigned u;
	std::set<IUnknown*>::iterator i;

	CComObject<tEnum>* t_pEnum=0;	// Pointer to our enumerator

	EXPECT(t_spIEnumUnknown->Reset());	// Start looking at each potential output	

	do
	{	CComPtr<IUnknown> t_spIUnknown;
		ULONG t_ulFetched=0; 
		ONFAILGO(t_spIEnumUnknown->Next(1UL,&t_spIUnknown,&t_ulFetched),t_hReturn,releaseset);	// Fetch potential input

		if (!t_ulFetched)
			break;

		CComPtr<IOTInput> t_spIOTInput;
		if (SUCCEEDED(t_spIUnknown->QueryInterface(&t_spIOTInput)))	// If it is an input
		{	CComPtr<IOTConnector> t_spIOTConnectorInput;
			ONFAILGO(t_spIOTInput->Input(&t_spIOTConnectorInput),t_hReturn,releaseset);	// Ask for the connected connector

			if (t_spIOTConnectorInput)	// Is it connected?
			{	
				CComPtr<IOpenTool> t_spIOpenTool;
				ONFAILGO(t_spIOTConnectorInput->OpenTool(&t_spIOpenTool),t_hReturn,releaseset);	// Ask for the input's OpenTool

				IUnknown* t_pIUnknownOpenToolInput;	
				ONFAILGO(t_spIOpenTool->QueryInterface(&t_pIUnknownOpenToolInput),t_hReturn,releaseset);	// Get it's unknown

				t_setIUnknown.insert(t_pIUnknownOpenToolInput);	// Record it
			}
		}
	}
	while (true);

	t_ppIUnknown=new IUnknown*[t_setIUnknown.size()];	// Allocate space (can't pass to Init() directly)
	if (!t_ppIUnknown)
	{	t_hReturn=E_OUTOFMEMORY;
		goto releaseset;
	}

	u=0;
	for (i=t_setIUnknown.begin(); i!=t_setIUnknown.end(); ++i)
		t_ppIUnknown[u++]=*i;

	CComObject<tEnum>::CreateInstance(&t_pEnum);	// Create an enumerator
	if (!t_pEnum)
	{	t_hReturn=E_OUTOFMEMORY;
		goto releasearray;
	}

	ONFAILGO(t_pEnum->Init(t_ppIUnknown,t_ppIUnknown+u,0,AtlFlagCopy),t_hReturn,releaseall);	// Initialise it

	t_hReturn=t_pEnum->QueryInterface(IOUT(a_ppIEnumUnknown));	// Return the pointer to the enumerator

releaseall:

	t_pEnum->Release();	// Release our copy of the enumerator we returned

releasearray:
	delete t_ppIUnknown;	// Release the array

releaseset:
	
	for (i=t_setIUnknown.begin(); i!=t_setIUnknown.end(); ++i)
		(*i)->Release();	// Release our copies of the Open Tools

	return t_hReturn;
}

// IEnumUnknown

STDMETHODIMP TrivialAdder::Next(/* [in] */ ULONG celt,/* [out] */ IUnknown __RPC_FAR *__RPC_FAR *rgelt,/* [out] */ ULONG __RPC_FAR *pceltFetched)
{	if (!m_pEnumConnectors)
		return E_UNEXPECTED;
	return m_pEnumConnectors->Next(celt,rgelt,pceltFetched);
}
    
STDMETHODIMP TrivialAdder::Skip(/* [in] */ ULONG celt)
{	if (!m_pEnumConnectors)
		return E_UNEXPECTED;
	return m_pEnumConnectors->Skip(celt);
}
    
STDMETHODIMP TrivialAdder::Reset(void)
{	if (!m_pEnumConnectors)
		return E_UNEXPECTED;
	return m_pEnumConnectors->Reset();
}

    
STDMETHODIMP TrivialAdder::Clone(/* [out] */ IEnumUnknown __RPC_FAR *__RPC_FAR *ppenum)
{	if (!m_pEnumConnectors)
		return E_UNEXPECTED;
	return m_pEnumConnectors->Clone(ppenum);
}

